// 
// Copyright (C) 2007, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Sample indicator demonstrating various DrawObject methods
    /// </summary>
    [Description("Sample indicator demonstrating various Draw Object methods")]
    public class SampleDrawObject : Indicator
    {
        #region Variables
		private int rectWidth	= 0;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose	= true;
            Overlay				= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			// When the close of the bar crosses above the SMA(20), draw a blue diamond
			if (CrossAbove(Close, SMA(20), 1))
			{
				/* Adding the 'CurrentBar' to the string creates unique draw objects because they will all have unique IDs
				Having unique ID strings may cause performance issues if many objects are drawn */
				DrawDiamond("Up Diamond" + CurrentBar, false, 0, SMA(20)[0], Color.Blue);
			}
			
			// But when the close crosses below the SMA(20), draw a magenta diamond
			else if (CrossBelow(Close, SMA(20), 1))
			{
				/* Adding the 'CurrentBar' to the string creates unique draw objects because they will all have unique IDs
				Having unique ID strings may cause performance issues if many objects are drawn */
				DrawDiamond("Down Diamond" + CurrentBar, false, 0, SMA(20)[0], Color.Magenta);
			}
			
			// Draw a blue-violet rectangle representing the latest uptrend based on the SMA(20)
			if (Close[0] >= SMA(20)[0])
			{
				/* Because the ID string is not unique for every rectangle, the latest draw object will replace the
				old draw object. The rectangle's starting bar is determined by the uptrend counter. */
				DrawRectangle("Rectangle", false, rectWidth, Close[rectWidth], 0, Close[0], Color.Blue, Color.BlueViolet, 5);
				
				// Counter that keeps track of the uptrend by incrementing
				rectWidth++;
			}
			
			// Reset the uptrend counter when trend has ended
			if (Close[0] < SMA(20)[0])
				rectWidth = 0;
        }
        #region Properties
        #endregion
    }
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SampleDrawObject[] cacheSampleDrawObject = null;

        private static SampleDrawObject checkSampleDrawObject = new SampleDrawObject();

        /// <summary>
        /// Sample indicator demonstrating various Draw Object methods
        /// </summary>
        /// <returns></returns>
        public SampleDrawObject SampleDrawObject()
        {
            return SampleDrawObject(Input);
        }

        /// <summary>
        /// Sample indicator demonstrating various Draw Object methods
        /// </summary>
        /// <returns></returns>
        public SampleDrawObject SampleDrawObject(Data.IDataSeries input)
        {
            if (cacheSampleDrawObject != null)
                for (int idx = 0; idx < cacheSampleDrawObject.Length; idx++)
                    if (cacheSampleDrawObject[idx].EqualsInput(input))
                        return cacheSampleDrawObject[idx];

            lock (checkSampleDrawObject)
            {
                if (cacheSampleDrawObject != null)
                    for (int idx = 0; idx < cacheSampleDrawObject.Length; idx++)
                        if (cacheSampleDrawObject[idx].EqualsInput(input))
                            return cacheSampleDrawObject[idx];

                SampleDrawObject indicator = new SampleDrawObject();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                SampleDrawObject[] tmp = new SampleDrawObject[cacheSampleDrawObject == null ? 1 : cacheSampleDrawObject.Length + 1];
                if (cacheSampleDrawObject != null)
                    cacheSampleDrawObject.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSampleDrawObject = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Sample indicator demonstrating various Draw Object methods
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleDrawObject SampleDrawObject()
        {
            return _indicator.SampleDrawObject(Input);
        }

        /// <summary>
        /// Sample indicator demonstrating various Draw Object methods
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleDrawObject SampleDrawObject(Data.IDataSeries input)
        {
            return _indicator.SampleDrawObject(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Sample indicator demonstrating various Draw Object methods
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleDrawObject SampleDrawObject()
        {
            return _indicator.SampleDrawObject(Input);
        }

        /// <summary>
        /// Sample indicator demonstrating various Draw Object methods
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleDrawObject SampleDrawObject(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SampleDrawObject(input);
        }
    }
}
#endregion
